*
*   File name:   lpeflpt1.asm
*
*   Description: filtro passa-basso ellittico realizzato in forma
*	parallela. Versione REAL-TIME con coefficienti NON SCALATI.
*	La realizzazione soffre di problemi di overflow ad alcune frequenze. Ad
*	esempio un segnale del tipo sin(1000Hz)+sin(3500Hz) viene correttamente
*	filtrato, mentre uno del tipo sin(500Hz)+sin(3500Hz) presenta delle spurie
*	a 1500Hz e 2500Hz.
*
*   Specifiche di progetto:
*
*		fp = 2 kHz	DBR < .5 dB
*		fs = 3 kHz	Att < 40 dB
*
*   Caratteristiche del filtro:
*
*		fp = 2kHz	DBR = .5 dB
*		fs = 3kHz	Att = 56 dB
*
*   Last mofified: 16/06/98
*

	.title   "Low pass elliptic filter: DBR = .1 dB, not scaled, real-time"
	.mmregs
	.width   80
	.length  60

	.setsect ".text",   1800h, 0
	.setsect ".data",   0200h, 1
	.setsect ".bss",    0210h, 1
	.setsect "vectors", 0180h, 0


	.sect "vectors"			; interrupt vector table stored at 
AC01_enable .set 1
	.copy "lpefvecs.asm"		; location 0x0180


	.sect ".data"
*
*	|------------------------------------->|
*	|                                      |
*	|                                      |
*   --->1----------->4----------->4--- b10 --->2--- G --->3
*	|            |            |            |
*	|            |            |            |
*	|            |<-- -a11 ---5--- b11 --->2
*	|            |            |            |
*	|            |            |            |
*	|            |<-- -a12 ---6            |
*	|                                      |
*	|                                      |
*	|----------->7----------->7--- b20 --->2
*	             |            |            |
*	             |            |            |
*	             |<-- -a21 ---8--- b21 --->|
*	             |            |
*	             |            |
*	             |<-- -a22 ---9
*

COEFF
	.word	-8417	; b11/2
	.word	24022	; a12
	.word	1735	; a11
	.word	-8526	; b10
	.word	23876	; b21/2
	.word	7519	; a22
	.word	-17633	; a21
	.word	-18729	; b20
	.word	16740	; G

STATUS	.word	1	; W11
	.word	2	; W12
	.word	3	; W21
	.word	4	; W22

ov4_counter		; contatore di overflow del nodo 4
	.word	0
ov7_counter		; contatore di overflow del nodo 7
	.word	0
ov2_counter		; contatore di overflow del nodo 2
	.word	0
end_data

inbuf	.set 2000h	; Per compatibilita` con la versione "not real-time"
length	.set 080h
outbuf	.set 2400h


	.sect ".text"
start:
	intm = 1		; disable all interrupts
	call AC01INIT		; initialize Analog interface.
	DP = #0
	nop
	pmst = #01a0h		; Interrupt pointer maps vectors to address 180h
	sp = #0ffah		; stack pointer located in Communications Kernal
	imr = #240h		; unmask TDM RINT and HPIINT(host port interface)
	FRCT = #1
	OVM = #0

	.asg AR1, in
	.asg AR2, out
	.asg AR3, coeff
	.asg AR4, status
	.asg AR5, ovc
	in = #inbuf
	out = #outbuf
	BK = #length
	status = #STATUS
	repeat(#3), A = 0	; resetta lo stato del filtro
	*status+ = A
	intm = 0		; enable all interrupts

WAIT:
	goto WAIT

receive:
	coeff  = #COEFF
	status = #STATUS
	ovc = #ov4_counter
	A = trcv
	*+in(1)% = A				; (estensione di segno)

	B = *in<<16			; y(n)=x(n)
	@BRC = #1			; due celle di 2o grado
	blockrepeat(end_cell)
begin_cell
        B += *coeff * *status		; y(n)+=(bi1/2)*Wi1
        B += *coeff+ * *status+		; y(n)+=(bi1/2)*Wi1
	A = *in<<16			; yi=x(n)
	A -= *coeff+ * *status-		; yi-=ai2*Wi2
	A = rnd(A - *status * *coeff+)	; yi-=ai1*Wi1 (T=Wi1)
	if (AOV) execute(1)
	   *ovc += 1
	*ovc+ += 0			; (serve solo per incrementare il puntatore)
	*status+ = hi(A)		; Wi1=yi
	*status+ = T			; Wi2=T
	B += *coeff+ * hi(A)		; y(n)+=bi0*yi
end_cell

	A = B				; Oss.: errore di troncamento. Il
	B = *coeff * hi(A)		; processore non supporta
					; l'istruzione `A = rnd(B)'
	if (BOV) execute(1)
	   *ovc += 1

	A = B & #~11b<<16
	*out+% = hi(A)
	@tdxr = hi(A)
	return_enable


transmit:
	return_enable


	.copy "acinit.asm"

	.end
