*
*   File name:   lpeflpt2.asm
*
*   Description: filtro passa-basso ellittico realizzato in forma parallela.
*	Versione NON REAL-TIME con coefficienti SCALATI in norma l1.
*
*	Per ottenere una convincente simulazione del corretto
*	funzionamento del filtro impostare i seguenti probe-point:
*	- caricare "length+1" campioni del file del segnale di ingresso
*	di periodo multiplo di "length" all'indirizzo "inpp";
*	- visualizzare un grafico all'indirizzo "inpp" con "Buffer size"
*	e "Display size" pari a "length";
*	- visualizzare un grafico all'indirizzo "outpp" con "Buffer size"
*	e "Display size" pari a "length".
*
*
*   Specifiche di progetto:
*
*		fp = 2 kHz	DBR < .5 dB
*		fs = 3 kHz	Att < 40 dB
*
*   Caratteristiche del filtro:
*
*		fp = 2 kHz	DBR < .1 dB
*		fs = 3 kHz	Att < 49 dB
*
*
*   Last mofified: 18/06/98
*

	.title   "Low pass elliptic filter: DBR = .1 dB, scaled, not real-time, l1"
	.mmregs
	.width   80
	.length  60

	.setsect ".text",   1800h, 0
	.setsect ".data",   0200h, 1
	.setsect ".bss",    0210h, 1
	.setsect "vectors", 0180h, 0


	.sect "vectors"			; interrupt vector table stored at
AC01_enable .set 0
	.copy "lpefvecs.asm"		; location 0x0180

	.sect ".data"

*
*	|----------------- 1/2 --------------->|
*	|                                      |
*	|                                      |
*   --->1--- 1/4 --->4----------->4--- b'10 -->2--- G' -->3
*	|            |            |            |
*	|            |            |            |
*	|            |<-- -a11 ---5--- b'11 -->2
*	|            |            |            |
*	|            |            |            |
*	|            |<-- -a12 ---6            |
*	|                                      |
*	|                                      |
*	|--- 1/2 --->7----------->7--- b20 --->2
*	             |            |            |
*	             |            |            |
*	             |<-- -a21 ---8--- b21 --->|
*	             |            |
*	             |            |
*	             |<-- -a22 ---9
*

COEFF
	.word	-7316*2
	.word	21792
	.word	7248
	.word	-4900*2
	.word	20442
	.word	4090
	.word	-6616
	.word	-25148
	.word	25041
	; b11 a12 a11 b10
	; b21 a22 a21 b20
	; G/2

STATUS	.word	0			; W11
	.word	0			; W12
	.word	0			; W21
	.word	0			; W22


inbuf	.set 2000h
length	.set 020h
outbuf	.set 2400h


	.sect ".text"
start:

	intm = 1		; disable all interrupts
	DP = #0
	nop
	pmst = #01a0h		; Interrupt pointer maps vectors to address 180h
	sp = #0ffah		; stack pointer located in Communications Kernal
	imr = #240h		; unmask TDM RINT and HPIINT(host port interface)
	FRCT = #1
	intm = 0		; enable all interrupts


	.asg AR1, in
	.asg AR2, out
	.asg AR3, coeff
	.asg AR4, status
	in = #inbuf
	BK = #length
LOOP:
	coeff  = #COEFF
	status = #STATUS

inpp
	B = *in<<(16-1)			; y(n)=x(n)/2

first_cell
        B += *coeff+ * *status+		; y(n)+=(2*b11)*W11
	A = *in<<(16-2)			; y'=x(n)/4
	A -= *coeff+ * *status-		; y'-=a22*W22
	A = rnd(A - *status * *coeff+)	; y'-=a21*W21 (T=W21)
	*status+ = hi(A)		; W21=y'
	*status+ = T			; W22=T
	B += *coeff+ * hi(A)		; y(n)+=(2*b20)*y'

second_cell
        B += *coeff+ * *status+		; y(n)+=b21*W21
	A = *in<<(16-1)			; y'=x(n)/2
	A -= *coeff+ * *status-		; y'-=a22*W22
	A = rnd(A - *status * *coeff+)	; y'-=a21*W21 (T=W21)
	*status+ = hi(A)		; W21=y'
	*status+ = T			; W22=T
	B = rnd(B + *coeff+ * hi(A))	; y(n)+=b20*y'

out_cells
	A = B				; y'=y(n)
	B = *coeff * hi(A)		; y(n)=(G/2)*y'
	B += *coeff * hi(A)		; y(n)+=(G/2)*y'

	out= #outbuf
	repeat(#length-2)
	*out+ = data(outbuf+1)
	*out = B<<(-16+1)		; y(n)*=2
outpp
	goto LOOP

	.end
